@@ -1,4 +1,4 @@
-poisson.lerouxCAR <- function(formula, data=NULL,  W, burnin=0, n.sample=1000, thin=1, blocksize.beta=5, prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, prior.rho=NULL, verbose=TRUE)
+poisson.lerouxCAR <- function(formula, data=NULL,  W, burnin=0, n.sample=1000, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, verbose=TRUE)
 {
 #### Check on the verbose option
      if(is.null(verbose)) verbose=TRUE     
@@ -95,12 +95,7 @@ offset <- try(model.offset(frame), silent=TRUE)
 beta <- glm(Y~X.standardised-1, offset=offset, family=poisson)$coefficients
 phi <- rnorm(n=n, mean=rep(0,n), sd=rep(0.1, n))
 tau2 <- runif(1)
-         
-    if(is.null(prior.rho)) prior.rho <- seq(0, 0.99, 0.01)
-    if(min(prior.rho) < 0 | max(prior.rho) >=1) stop("prior.rho is outside the interval [0,1).", call.=FALSE)
-    if(sum(duplicated(prior.rho))>0) stop("prior.rho has duplicate values.", call.=FALSE)     
-rho <- sample(prior.rho, size=1)
-which.rho <- which(rho==prior.rho)    
+rho <- runif(1)         
           
           
 #### Priors
@@ -135,13 +130,10 @@ which.rho <- which(rho==prior.rho)
      if(n.sample <= burnin)  stop("Burn-in is greater than n.sample.", call.=FALSE)
      if(!is.numeric(thin)) stop("thin is not a number", call.=FALSE)
      if(thin <= 0) stop("thin is less than or equal to zero.", call.=FALSE)
-
-     if(!is.numeric(blocksize.beta)) stop("blocksize.beta is not a number", call.=FALSE)
-     if(blocksize.beta <= 0) stop("blocksize.beta is less than or equal to zero", call.=FALSE)
-     if(!(floor(blocksize.beta)==ceiling(blocksize.beta))) stop("blocksize.beta has non-integer values.", call.=FALSE)
           
           
 ## Compute the blocking structure for beta     
+blocksize.beta <- 5 
      if(blocksize.beta >= p)
      {
      n.beta.block <- 1
@@ -174,7 +166,7 @@ samples.beta <- array(NA, c(n.keep, p))
 samples.phi <- array(NA, c(n.keep, n))
 samples.tau2 <- array(NA, c(n.keep, 1))
 samples.rho <- array(NA, c(n.keep, 1))
-samples.deviance <- array(NA, c(n.keep, n))
+samples.deviance <- array(NA, c(n.keep, 1))
 samples.fitted <- array(NA, c(n.keep, n))
           
 ## Metropolis quantities
@@ -186,7 +178,7 @@ proposal.sd.rho <- 0.02
 proposal.corr.beta <- solve(t(X.standardised) %*% X.standardised)
 chol.proposal.corr.beta <- chol(proposal.corr.beta) 
 tau2.posterior.shape <- prior.tau2[1] + 0.5 * n
-rho.step <- 5          
+       
           
           
 #### CAR quantities
@@ -200,41 +192,39 @@ rho.step <- 5
      if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
           
 
-### Create the duplet form
-n.neighbours <- as.numeric(apply(W, 1, sum))
-W.duplet <- c(NA, NA)
+## Create the triplet object
+W.triplet <- c(NA, NA, NA)
      for(i in 1:n)
      {
           for(j in 1:n)
           {
                if(W[i,j]==1)
                {
-               W.duplet <- rbind(W.duplet, c(i,j))     
+               W.triplet <- rbind(W.triplet, c(i,j, W[i,j]))     
                }else{}
           }
      }
-W.duplet <- W.duplet[-1, ]     
-n.duplet <- nrow(W.duplet) 
+W.triplet <- W.triplet[-1, ]     
+n.triplet <- nrow(W.triplet) 
+W.triplet.sum <- tapply(W.triplet[ ,3], W.triplet[ ,1], sum)
+n.neighbours <- tapply(W.triplet[ ,3], W.triplet[ ,1], length)
 
-
-## Create the list object
-Wlist <- as.list(rep(NA,n))     
+     
+## Create the start and finish points for W updating
+W.begfin <- array(NA, c(n, 2))     
+temp <- 1
      for(i in 1:n)
      {
-     Wlist[[i]] <- which(W[i, ]==1)     
+     W.begfin[i, ] <- c(temp, (temp + n.neighbours[i]-1))
+     temp <- temp + n.neighbours[i]
      }
-          
+     
              
-## Create the set of determinants     
-Wstar <- diag(n.neighbours) - W
+## Create the determinant     
+Wstar <- diag(apply(W,1,sum)) - W
 Wstar.eigen <- eigen(Wstar)
 Wstar.val <- Wstar.eigen$values
-n.rho <- length(prior.rho)
-det.Q <- rep(NA, n.rho)
-     for(i in 1:n.rho)
-     {
-     det.Q[i] <-  0.5 * sum(log((prior.rho[i] * Wstar.val + (1-prior.rho[i]))))    
-     }
+det.Q <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
 
      
      
@@ -285,7 +275,7 @@ det.Q <- rep(NA, n.rho)
      ## Sample from phi
      ####################
      beta.offset <- X.standardised %*% beta + offset
-     temp1 <- poissoncarupdate(W_list=Wlist, nsites=n, phi=phi, nneighbours=n.neighbours, tau2=tau2, y=Y, phi_tune=proposal.sd.phi, rho_num=rho, rho_den=rho, offset=beta.offset)
+     temp1 <- poissoncarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=n, phi=phi, tau2=tau2, y=Y, phi_tune=proposal.sd.phi, rho=rho, offset=beta.offset)
      phi <- temp1[[1]]
      phi <- phi - mean(phi)
      accept[3] <- accept[3] + temp1[[2]]
@@ -296,7 +286,7 @@ det.Q <- rep(NA, n.rho)
      ##################
      ## Sample from tau2
      ##################
-     temp2 <- quadform(W_duplet1=W.duplet[ ,1], W_duplet2=W.duplet[ ,2], n_duplet=n.duplet,  nsites=n, phi=phi, nneighbours=n.neighbours, diagonal=rho, offdiagonal=rho)      
+     temp2 <- quadform(W.triplet, W.triplet.sum, n.triplet, n, phi, phi, rho)
      tau2.posterior.scale <- temp2 + prior.tau2[2] 
      tau2 <- 1 / rgamma(1, tau2.posterior.shape, scale=(1/tau2.posterior.scale))
               
@@ -305,18 +295,18 @@ det.Q <- rep(NA, n.rho)
      ##################
      ## Sample from rho
      ##################
-     which.proposal.rho <- sample(setdiff(max(1, which.rho-rho.step):min(which.rho+rho.step, n.rho), which.rho), size=1)    
-     proposal.rho <- prior.rho[which.proposal.rho]
-     temp3 <- quadform(W_duplet1=W.duplet[ ,1], W_duplet2=W.duplet[ ,2], n_duplet=n.duplet,  nsites=n, phi=phi, nneighbours=n.neighbours, diagonal=proposal.rho, offdiagonal=proposal.rho)      
-     logprob.current <- det.Q[which.rho] - temp2 / tau2
-     logprob.proposal <- det.Q[which.proposal.rho] - temp3 / tau2
+     proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1, mean=rho, sd=proposal.sd.rho)   
+     temp3 <- quadform(W.triplet, W.triplet.sum, n.triplet, n, phi, phi, proposal.rho)
+     det.Q.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))              
+     logprob.current <- det.Q - temp2 / tau2
+     logprob.proposal <- det.Q.proposal - temp3 / tau2
      prob <- exp(logprob.proposal - logprob.current)
     
      #### Accept or reject the proposal
          if(prob > runif(1))
          {
          rho <- proposal.rho
-         which.rho <- which.proposal.rho
+         det.Q <- det.Q.proposal
          accept[5] <- accept[5] + 1           
          }else
          {
@@ -328,7 +318,8 @@ det.Q <- rep(NA, n.rho)
      ## Calculate the deviance
      #########################
      fitted <- exp(as.numeric(X.standardised %*% beta) + phi + offset)
-     deviance <- dpois(x=as.numeric(Y), lambda=fitted)
+    deviance.all <- dpois(x=as.numeric(Y), lambda=fitted, log=TRUE)
+    deviance <- -2 * sum(deviance.all)  
                
                
                
@@ -387,10 +378,10 @@ det.Q <- rep(NA, n.rho)
           #### rho tuning parameter
                if(accept.rho > 70)
                {
-               rho.step <- min(rho.step+1, round(n.rho/4))
+               proposal.sd.rho <- min(2 * proposal.sd.rho, 0.5)
                }else if(accept.rho < 50)              
                {
-               proposal.sd.rho <- max(rho.step-1, 1)
+               proposal.sd.rho <- 0.5 * proposal.sd.rho
                }else
                {
                }
@@ -435,17 +426,17 @@ median.beta <- apply(samples.beta, 2, median)
 median.phi <- apply(samples.phi, 2, median)
 fitted.median <- exp(X.standardised %*% median.beta + median.phi + offset)
 deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE))
-deviance.sum <- apply(-2 * log(samples.deviance), 1, sum)
-p.d <- mean(deviance.sum) - deviance.fitted
-DIC <- 2 * mean(deviance.sum) - deviance.fitted
-like.fitted <- apply(samples.deviance, 2, mean)
-DIC3 <- 2 * mean(deviance.sum)   + 2 * sum(log(like.fitted))     
+p.d <- median(samples.deviance) - deviance.fitted
+DIC <- 2 * median(samples.deviance) - deviance.fitted     
 
      
 #### Compute the Conditional Predictive Ordinate
-CPO.temp <- 1 / samples.deviance
-CPO <- 1/apply(CPO.temp, 2, mean)
-MPL <- sum(log(CPO))   
+CPO <- rep(NA, n)
+     for(j in 1:n)
+     {
+     CPO[j] <- 1/median((1 / dpois(x=Y[j], lambda=samples.fitted[ ,j])))    
+     }
+LMPL <- sum(log(CPO))  
           
           
 #### transform the parameters back to the origianl covariate scale.
@@ -498,32 +489,18 @@ summary.results[ , 4:5] <- round(summary.results[ , 4:5], 1)
           
                    
 #### Create the Fitted values and residuals
-fitted.values <- array(NA, c(n, 5))
-colnames(fitted.values) <- c("Mean", "Sd", "Median", "2.5%", "97.5%")
-fitted.values[ ,1] <- apply(samples.fitted, 2, mean)
-fitted.values[ ,2] <- apply(samples.fitted, 2, sd)
-fitted.values[ ,3:5] <- t(apply(samples.fitted, 2, quantile, c(0.5, 0.025, 0.975)))
-fitted.values <- round(fitted.values, 4)
-
-residuals <- array(NA, c(n, 5))
-colnames(residuals) <- c("Mean", "Sd", "Median", "2.5%", "97.5%")
-residuals.temp <- array(NA, c(nrow(samples.beta), n))
-     for(i in 1:nrow(samples.beta))
-     {
-     residuals.temp[i, ] <- as.numeric(Y) - samples.fitted[i, ]
-     }
-residuals[ ,1] <- apply(residuals.temp, 2, mean)
-residuals[ ,2] <- apply(residuals.temp, 2, sd)
-residuals[ ,3:5] <- t(apply(residuals.temp, 2, quantile, c(0.5, 0.025, 0.975)))
-residuals <- round(residuals, 4)
-
+fitted.values <- apply(samples.fitted, 2, median)
+residuals <- as.numeric(Y) - fitted.values
 
+     
 ## Compile and return the results
-modelfit <- c(DIC, p.d, DIC3, MPL)
-names(modelfit) <- c("DIC", "p.d", "DIC3", "MPL")
+modelfit <- c(DIC, p.d, LMPL)
+names(modelfit) <- c("DIC", "p.d", "LMPL")
 model.string <- c("Likelihood model - Poisson (log link function)", "\nRandom effects model - Leroux CAR\n")
-samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), rho=mcmc(samples.rho))
-results <- list(formula=formula,  samples=samples, fitted.values=fitted.values, residuals=residuals, W.summary=W, modelfit=modelfit,  summary.results=summary.results, model=model.string, accept=accept.final)
+samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), tau2=mcmc(samples.tau2), rho=mcmc(samples.rho), fitted=mcmc(samples.fitted))
+results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL,  formula=formula, model=model.string, X=X)
+
+     
 class(results) <- "carbayes"
 
      if(verbose)
